/**
 * @author Youtiao 2016/9/26
 */

/**
 * 日期格式化
 * @param {string} format 格式，例如'yyyyMMhh'
 * @param {string} [lang] 语言，例如'en'，'zh'
 */
Date.prototype.format = function (format, lang) {
    var _this = this;

    var ARR_MONTH = [];
    var ARR_DAY_OF_WEEK = [];
    switch (lang) {
        case 'en':
            // 英文
            ARR_MONTH = ['January','February','March','April','May','June','July','August','September','October','November','December'];
            ARR_DAY_OF_WEEK = ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'];
            break;
        case 'zh':
        default:
            // 默认中文
            ARR_MONTH = ['一','二','三','四','五','六','七','八','九','十','十一','十二'];
            ARR_DAY_OF_WEEK = ['一','二','三','四','五','六','日'];
    }

    /**
     * 补零
     * @param value 需要补零的对象
     * @param {int} length 补足后的最终长度
     * @return {string}
     */
    var zeroize = function (value, length) {
        // length默认值为2
        if (!length) length = 2;
        // value转换为字符串
        value = String(value);
        // 需要补足的'0'
        var zeros = '';
        for (var i = 0; i < (length - value.length); i++) zeros += '0';

        return zeros + value;
    };

    /**
     * 格式化时区
     * @param {Date} objDate 时间对象
     * @returns {string} 时区字符串，例如'+8000'
     */
    var strTimezone = function (objDate) {
        // 时差
        var timezoneOffset = objDate.getTimezoneOffset();
        // 小时
        var h = parseInt(Math.abs(timezoneOffset)/60);
        // 分钟
        var m = Math.abs(timezoneOffset)%60;
        var strTimezone = zeroize(h) + zeroize(m);

        return timezoneOffset<=0 ? '+'+strTimezone : '-'+strTimezone;
    };

    /**
     * /"[^"]*"|'['^]*'|(?:d{1,4}|m{1,4}|yy(?:yy)?|([hHMstT])\1?|[lLZ])/g
     * "[^"]*"|'['^]*'，双引号和单引号必须成对出现（两个引号执之间可以是除引号以外的任意符号）
     * yy(?:yy) 匹配'yy'或者'yyyy'，(?:yy)为非捕获组
     * \1 第一个捕获组
     * ([hHMstT])\1?  =>  ([hHMstT])([hHMstT])?
     * Fri Sep 08 2016 16:44:18 GMT+0800 (中国标准时间)
     */
    return format.replace(/"[^"]*"|'['^]*'|(?:d{1,4}|m{1,4}|M{1,4}|yy(?:yy)?|([hHsS])\1?|[aAzZ])/g, function (match) {
        switch (match) {
            case 'yy':
                // 两位数的年份，例如16
                return String(_this.getFullYear()).substr(2);
            case 'yyyy':
                // 四位数的年份，例如2016
                return _this.getFullYear();
            case 'M':
                // 月份数字形式，例如9
                return _this.getMonth() + 1;
            case 'MM':
                // 月份数字形式补零，例如09
                return zeroize(_this.getMonth() + 1);
            case 'MMM':
                // 月份文字形式缩写
                return ARR_MONTH[_this.getMonth()].substr(0, 3);
            case 'MMMM':
                // 月份文字形式
                return ARR_MONTH[_this.getMonth()];
            case 'd':
                // 月份中的日期，例如1-31
                return _this.getDate();
            case 'dd':
                // 月份中的日期补零，例如01-31
                return zeroize(_this.getDate());
            case 'ddd':
                // 星期中的天数文字形式缩写
                return ARR_DAY_OF_WEEK[_this.getDay() - 1].substr(0, 3);
            case 'dddd':
                // 星期中的天数文字形式
                return ARR_DAY_OF_WEEK[_this.getDay() - 1];
            case 'h':
                // 12小时制的小时，例如1-12
                return _this.getHours() % 12 || 12;
            case 'hh':
                // 12小时制的小时补零，例如01-12
                return zeroize(_this.getHours() % 12 || 12);
            case 'H':
                // 24小时制的小时，例如0-23
                return _this.getHours();
            case 'HH':
                // 24小时制的小时补零，例如00-23
                return zeroize(_this.getHours());
            case 'm':
                // 分钟数，例如0-59
                return _this.getMinutes();
            case 'mm':
                // 分钟数补零，例如00-59
                return zeroize(_this.getMinutes());
            case 's':
                // 秒数，例如0-59
                return _this.getSeconds();
            case 'ss':
                // 秒数补零，例如00-59
                return zeroize(_this.getSeconds());
            case 'S':
                // 毫秒数，例如0-999
                return _this.getMilliseconds();
            case 'SS':
                // 毫秒数补零，例如000-999
                return zeroize(_this.getMilliseconds(), 3);
            case 'a':
                // 小写的am/pm 标记
                return _this.getHours() < 12 ? 'am' : 'pm';
            case 'A':
                // 大写的am/pm 标记
                return _this.getHours() < 12 ? 'AM' : 'PM';
            case 'z':
                // 时区，例如'GMT'
                return _this.toUTCString().match(/[A-Z]+$/);
            case 'Z':
                // 时区，例如'+0800'
                return strTimezone(_this);
            default:
                return match;
        }
    });
};
